/* December 2011 */
/* simplest model with no AR */
/* This program calculates the empirical critical values for both supTS and expTS with 3000 Monte Carlo simulations */
/* expTS is the avgEXP version  */
/* Critical values are used for getting size-corrected powers in Table 2 in the resubmission */

new;
st = hsec;
library pgraph;
graphset;
rndseed 2938438;

alpha = 0;

a = 1;
t = 200;     @sample size@

rho_b = 0.7;    @upper bound for rho@

itn = 3000;   @iterations we simulate the data under H0@

eps = rndn(t, itn);

supts = zeros(itn,1);	@stores the supTS test statistic for each iteration@ 
expts = zeros(itn,1);	@stores the expTS test statistic for each iteration@


for it(1,itn,1);

ys = eps[.,it];

/* MLE under H0 */

u_h = ys - meanc(ys);
sig_h = sqrt(meanc(u_h ^ 2));


rho = -rho_b;   @lower bound of rho@
ir = 1;           @iterations for rho@

cv = zeros(200,1); @stores supTS asymptotic critical values for each rho@

cv2 = zeros(200,1);    @stores expTS asymptotic critical values for each rho@

do while rho<=rho_b+0.01;

/* calculate our test statistic supTS and expTS */

lt1 = u_h/sig_h^2 ~ (u_h^2-sig_h^2)/(2*sig_h^4); @1st order derivative of log likelihood@


mu2t = zeros(t,1);

/* Calculate the cross-product of u_t and u_s with single loop */

xs = zeros(t,1);

xs[2] = rho * u_h[1];

mu2t[2] = u_h[2] * xs[2];

for i(3,t,1);
    xs[i] = rho * (xs[i-1] + u_h[i-1]);
    mu2t[i] = u_h[i] * xs[i];
endfor;

gamma_e = 1/sqrt(t)*sumc(mu2t)/sig_h^4;

mu2t = mu2t/sig_h^4 + (u_h^2-sig_h^2)/(2*sig_h^4);

epsilont = mu2t-lt1*inv(lt1'lt1)*lt1'mu2t; @projection of mu2t on lt1@

esqe = meanc(epsilont^2);

tspe = gamma_e/sqrt(esqe);  @test statistic process@

if esqe < 1e-5;
    cv[ir] = 0;
    cv2[ir] = 1;
else;
    cv[ir] = 1/2*maxc(tspe|0)^2;    @supTS test statistic@ 

    cv2[ir] =sqrt(2*pi)*exp((tspe-1)^2/2)*cdfn(tspe-1);  @expTS test statistic@
endif;

rho = rho + 0.01;
ir = ir + 1;

endo;

cv = cv[1:ir-1];

cv2 = cv2[1:ir-1];

supts[it] = maxc(cv);   @supTS test statistic@
expts[it] = meanc(cv2); @expTS test statistic@

endfor;

supts = sortc(supts,1);
expts = sortc(expts,1);

crtsup99 = supts[floor(itn*0.99)];
crtsup95 = supts[floor(itn*0.95)];
crtsup90 = supts[floor(itn*0.90)];

crtexp99 = expts[floor(itn*0.99)];
crtexp95 = expts[floor(itn*0.95)];
crtexp90 = expts[floor(itn*0.90)];



format /rd 6,3;
"";
"supTS critical values are ";
"";
"1%       5%     10%";
crtsup99 crtsup95 crtsup90;
"";
"expTS critical values are ";
"";
"1%       5%     10%";
crtexp99 crtexp95 crtexp90;
"";
"";
"time elapsed " etstr(hsec-st);
 
end;

