/* December 2011 */
/* simplest model with no AR */
/* This program calculates the empirical powers for both supTS and expTS with 3000 Monte Carlo simulations */
/* expTS is the avgEXP version  */
/* Critical values are calculated from chp_CV.g  */
/* (p, q) = (0.75, 0.75), (0.25, 0.25), (0.9, 0.1), change trs correspondingly */
/* Table 2 in the resubmission */

new;
st = hsec;
library pgraph;
graphset;
rndseed 2938438;

alpha = 0;

a = 1;
t = 200;     @sample size@

maxs = 4; 
ksi={1 -1};
trs = {0.9 0.1, 0.5 0.5};
rho_b = 0.7;    @upper bound for rho@

itn = 3000;   @iterations we simulate the data under H0@

eps = rndn(t, itn);

supts = zeros(itn,1);	@stores the supTS test statistic for each iteration@ 
expts = zeros(itn,1);	@stores the expTS test statistic for each iteration@

/* critical values are calculated using chp_CV.g */


crtsup99 = 4.094;
crtsup95 = 2.508;
crtsup90 = 1.856;

crtexp99 = 4.512;
crtexp95 = 1.834;
crtexp90 = 1.362;

/* now to calculate the power */

supp = zeros(itn,4);    @supTS test statistic@
expp = zeros(itn,4);    @expTS test statistic@

for it(1,itn,1);

/* simulate the series. 200 more observations are simulated and then the first 200 are discarded */
@s0 denotes the initial state@
s0u = rndu(1,1);
s0 = 1*(s0u<0.5)+2*(s0u>0.5);

alt=ksi[s0]~ksi*markov(trs,t+200,s0);

alt = alt';
altn = alt[201:200+t];

si = 1;

do while si <= 4;

/* simulate y */

ys = a * si * altn/t^0.25 + eps[.,it];

/* MLE under H0 */

u_h = ys - meanc(ys);
sig_h = sqrt(meanc(u_h ^ 2));


rho = -rho_b;   @lower bound of rho@
ir = 1;           @iterations for rho@

cv = zeros(200,1); @stores supTS asymptotic critical values for each rho@

cv2 = zeros(200,1);    @stores expTS asymptotic critical values for each rho@

do while rho<=rho_b+0.01;

/* calculate our test statistic supTS and expTS */

lt1 = u_h/sig_h^2 ~ (u_h^2-sig_h^2)/(2*sig_h^4); @1st order derivative of log likelihood@


mu2t = zeros(t,1);

/* Calculate the cross-product of u_t and u_s with single loop */

xs = zeros(t,1);

xs[2] = rho * u_h[1];

mu2t[2] = u_h[2] * xs[2];

for i(3,t,1);
    xs[i] = rho * (xs[i-1] + u_h[i-1]);
    mu2t[i] = u_h[i] * xs[i];
endfor;

gamma_e = 1/sqrt(t)*sumc(mu2t)/sig_h^4;

mu2t = mu2t/sig_h^4 + (u_h^2-sig_h^2)/(2*sig_h^4);

epsilont = mu2t-lt1*inv(lt1'lt1)*lt1'mu2t; @projection of mu2t on lt1@

esqe = meanc(epsilont^2);

tspe = gamma_e/sqrt(esqe);  @test statistic process@

if esqe < 1e-5;
    cv[ir] = 0;
    cv2[ir] = 1;
else;
    cv[ir] = 1/2*maxc(tspe|0)^2;    @supTS test statistic@ 

    cv2[ir] = sqrt(2*pi)*exp((tspe-1)^2/2)*cdfn(tspe-1);  @expTS test statistic@
endif;

rho = rho + 0.01;
ir = ir + 1;

endo;

cv = cv[1:ir-1];

cv2 = cv2[1:ir-1];

supp[it,si] = maxc(cv);   @supTS test statistic@
expp[it,si] = meanc(cv2); @expTS test statistic@

si = si + 1;
endo;

endfor;
suppow99 = meanc(supp.>crtsup99)*100;
suppow95 = meanc(supp.>crtsup95)*100;
suppow90 = meanc(supp.>crtsup90)*100;

exppow99 = meanc(expp.>crtexp99)*100;
exppow95 = meanc(expp.>crtexp95)*100;
exppow90 = meanc(expp.>crtexp90)*100;

format /rd 8,3;
"";
"supTS critical values are ";
"";
"1%       5%     10%";
crtsup99 crtsup95 crtsup90;
"";
"expTS critical values are ";
"";
"1%       5%     10%";
crtexp99 crtexp95 crtexp90;
"";
"transition matrix is      " trs;
"";
"supTS powers are          ";
"";
"1%       5%     10%";
suppow99~suppow95~suppow90;
"";
"expTS powers are          ";
"";
"1%       5%     10%";
exppow99~exppow95~exppow90;

"";
"time elapsed " etstr(hsec-st);
 
end;

/* The procedure to get a Markov chain */

proc(1)=markov(trs,t,s0);	
/*  trs is the transition matrix
     t is the number of periods to simulate
     s0 is the initial state   */

local r,c,x,s,state,cum,k,ppi,ks;
r=rows(trs);
c=cols(trs);
x=rndu(t-1,1);
s=zeros(r,1);
s[s0]=1;

state=zeros(r,t-1); @state is a matrix recording the number of the realized state at time t @

cum=trs*upmat(ones(r,c));

for k(1,t-1,1);
	state[.,k]=s;
	ppi=0~s'*cum;
	s=((x[k].<=ppi[2:r+1]).*(x[k].>ppi[1:r]))';
endfor;

@
chain=ksi*state; chain is a row vector which stores the (T-1) period simulations@

retp(state);
endp;