/* December 2011 */
/* tabulate asymptotic critical values of supTS and expTS for the simplest model */
/* rho over [-0.7, 0.7] */
/* Table A1 in the resubmission  */

new;
st = hsec;

rndseed 2938438;

itn = 100000;
tr = 500;   @truncation at tr for the infinite sum@
rho_b = 0.7;   @lower bound for rho@


crt1 = zeros(itn,1);  @stores the supTS statistic@
crt2 = zeros(itn,1);  @stores the expTS statistic@

z = rndn(itn,tr+1);
/*sut=zeros(itn,1); */

for i(1,itn,1);
    rho = -rho_b;

    cv = zeros(200,1); @stores supTS asymptotic critical values for each rho@
    cv2 = zeros(200,1);    @stores expTS asymptotic critical values for each rho@
    irho = 1;
    do while rho <= rho_b+0.01;
        sign=(rho.>0)-(rho.<0); /* sign=1 if pos, =-1 if neg, =0 if null */
        su = z[i,1]; 
        for r(1,tr,1);
            su = su + (rho^r)*z[i,r+1];
        endfor;
        
        su = sign*su*sqrt(1-rho^2);
/*
       if rho^2/(1-rho^2)<1e-5;
   
        cv[irho] = 0;
       cv2[irho] = 1;
	else;
*/


	cv[irho] = 1/2*maxc(su|0)^2;    @supTS test statistic@ 
	cv2[irho] = sqrt(2*pi)*exp((su-1)^2/2)*cdfn(su-1); @expTS test statistic@

/*print "rho " rho;
print " cv2 " cv2[irho];
    endif; */

	 irho = irho + 1;
        rho = rho + 0.01;
    endo;

/*    sut[i]=su;*/

    cv = cv[1:irho-1];
    cv2 = cv2[1:irho-1];
  

    crt1[i] = maxc(cv);
    crt2[i] = meanc(cv2);
endfor;

crt1=sortc(crt1,1);
crt2=sortc(crt2,1);

format /rd 8,2;
print " 99% 95% 90% 80% 70% 50% 10% 5% 1%";
print " supTS ExpTS asymptotic critical values";
print crt1[floor(itn*0.99)]~crt2[floor(itn*0.99)];
print crt1[floor(itn*0.95)]~crt2[floor(itn*0.95)];
print crt1[floor(itn*0.90)]~crt2[floor(itn*0.90)];
print crt1[floor(itn*0.80)]~crt2[floor(itn*0.80)];
print crt1[floor(itn*0.70)]~crt2[floor(itn*0.70)];
print crt1[floor(itn*0.50)]~crt2[floor(itn*0.50)];
print crt1[floor(itn*0.10)]~crt2[floor(itn*0.10)];
print crt1[floor(itn*0.05)]~crt2[floor(itn*0.05)];
print crt1[floor(itn*0.01)]~crt2[floor(itn*0.01)];


"time elapsed " etstr(hsec-st);
end;
