/*  December 2011 */
/* Estimate the MS model for extended GNP series, adapted from Hansen's code */
/* Switching in the mean and the variance */
/*
The following parameters need to be specified:

dat = data
nar = 4            sets number of autoregressive lags

_gopt  = 1  to calculate global optimum

One needs to specify starting values for the parameters in  "th".  
The order is

(1) Mean in state 0
(2) AR coefficients
(3) Standard deviation of error in state 0 
(4) Difference in mean between states
(5) Standard deviation of error in state 1
(6) p
(7) q
*/
/* Table A3 in the resubmssion */

new;
output file = "markovm.out" reset;
output off;
n = 239;
load gnp[]=c:/paper/werner/revision/application/hamilton/gnpc96.txt;
gnp = 100*ln(gnp);
dat = gnp[2:n+1] - gnp[1:n];

nar = 4;    @ number of lags @
_gopt = 1;

let th = -.4 .01 -.06 -.25 -.21 .77  1.5 .77 2.2 1.1 ; @initial values for AR(4) @     

/*
let th = -.4 .01 .01 -.06 -.25 -.21 .77  1.5 .77 2.2 1.1 ; @initial values for AR(5)@  */

/*
let th = -.4 -.06 -.25 -.21 .77  1.5 .77 2.2 1.1 ; @initial values for AR(3) @     
*/

/*let th = -.4 -.25 -.21 .77  1.5 .77 2.2 1.1 ; @initial values for AR(2) @     
*/



library optmum;
optset;
#include optmum.ext;
_opmiter = 500;
_opgtol = .0001;
__output = 0;


y = dat[1+nar:n];
if nar .> 0;
  x = dat[nar:n-1];
  it = 2; 
do while it <= nar;
    x = x~dat[nar-it+1:n-it];
  it = it + 1; 
endo;
  z1 = 0|1;
  n3 = 2.^(nar-1);
  ss = z1.*.ones(n3,1);
  it = 2; 
do while it <= nar;
    ss = ss~(ones(2.^(it-1),1).*.(z1.*.ones(2.^(nar-it),1)));
  it = it + 1; 
endo;
  ss0 = ss[1:n3,.];
  ss1 = ss[n3+1:n3*2,.];
endif;



@  Calculation Under null  @

if nar .> 0;
  xx = ones(n-nar,1)~x;
else;
  xx = ones(n-nar,1);
endif;
xxi = invpd(moment(xx,0));
b0 = xxi*(xx'y);
u = y - xx*b0;
v0 = sqrt((u'u)/rows(u));
var0 = xxi*((xx.*u)'(xx.*u))*xxi;
se0 = sqrt(diag(var0));
if nar .> 0;
 phi = b0[2:1+nar];
 a0 = b0[1]/(1-sumc(phi));
 b = a0|phi|v0;
else;
 a0 = b0[1];
  b = a0|v0;
endif;
b1 = 0; p = 0.5; q = 0.5; sig1 = v0;
null = clike(b);


@  Calculation Under alternative  @
if _gopt == 1;

t3 = hsec;


__output = 0;
{bnew,f,gr,retcode} = optmum(&mglike,th);
lr = sumc(null) - f;

 g = gradp(&glike,bnew);
 s = g'g;
 h = pinv(hessp(&mglike,bnew)) ;
 v = h*s*h;

 p = bnew[5+nar];
 q = bnew[6+nar];
 p = exp(p)/(1+exp(p));
 q = exp(q)/(1+exp(q));
 bopt = bnew[1:4+nar]|p|q;
 vt = (p*(1-p))|(q*(1-q));
 v[5+nar 6+nar,5+nar 6+nar] = vt.*v[5+nar 6+nar,5+nar 6+nar].*(vt');

 se = sqrt(diag(v));
 g = cumsumc(g);
 stable = sumc(diag((g'g)/s))/rows(g);

t4 = hsec;
endif;

output on;
format /rd 8,4;
"Estimation Under Null of Linearity";
" Regression Parameters:"
b0~se0;
"";
"Standard Deviation : " v0;
"";
"Log Likelihood " sumc(null);
"";
if _gopt == 1;
"Results from estimation and testing of Markov switching model";
"";
"Estimates and standard errors";
"";
"Regression Parameters in State 0";
bopt[1:1+nar]~se[1:1+nar];
"Standard deviation in State 0";
bopt[2+nar]~se[2+nar];
"";
"Difference in mean between states";
bopt[3+nar]~se[3+nar];
"Standard deviation in State 1";
bopt[4+nar]~se[4+nar];
"P";
bopt[5+nar]~se[5+nar];
"Q";
bopt[6+nar]~se[6+nar];
"";"";
"Likelihood Value " f;
"AIC              " 2*f-2*cols(th);
"BIC              " 2*f-cols(th)*ln(n+1);
"Likelihood Ratio " lr;
"2*LR             " 2*lr;
"Nyblom Stability " stable;
"Estimation Time (seconds) " (t4-t3)/100;
"";"";
endif;
output off;
end;

proc marklike(th);
local c,alpha0,phi,sig0,alpha1,sig1,p,q,rho,b8,pxa,pxb,pa,pb,pp,z0,z1,p0,p1,
      q00,q01,q10,q11,f00,f01,f10,f11,fit,it,ff,f,nr,s1,sphi0,sphi1,z;

c = sqrt(2*pi);
alpha0 = th[1];
sig0 = th[2+nar];
alpha1 = th[3+nar];
sig1 = th[4+nar];
p = th[5+nar];
q = th[6+nar];

rho = (1-q)/(2-p-q);

if nar .> 0;
  phi = th[2:1+nar];
  b8 = alpha0*(1 - sumc(phi));
  sphi0 = ss0*phi*alpha1;
  sphi1 = ss1*phi*alpha1;
else;
  b8 = alpha0;
endif;

pp = (1-rho)|rho;
pxa = q|(1-p);
pxb = 1-pxa;

it = 2; 
do while it <= nar;
  nr = ones((2.^(it-2)),1);
  pp = (pp.*(pxa.*.nr))|(pp.*(pxb.*.nr));
it = it + 1; 
endo;
nr = rows(pp)/2;
p0 = pp[1:nr];
p1 = pp[nr+1:2*nr];

fit = zeros(n-nar,1);
it = 1;
do until it > n-nar;

  z = y[it] - b8;
  if nar .> 0;
    z = z - x[it,.]*phi;
    z0 = z + sphi0;
    z1 = z + sphi1;
  else;
    z0 = z;
    z1 = z;
  endif;

  f00 = exp(- (z0.^2)/(2*sig0*sig0))./(sig0*c);
  f01 = exp(- (z1.^2)/(2*sig0*sig0))./(sig0*c);
  f10 = exp(- ((z0-alpha1).^2)/(2*sig1*sig1))./(sig1*c);
  f11 = exp(- ((z1-alpha1).^2)/(2*sig1*sig1))./(sig1*c);
 
  q00 = (f00.*p0)*q;
  q01 = (f01.*p1)*(1-p);

  q10 = (f10.*p0)*(1-q);
  q11 = (f11.*p1)*p;

  if nar .> 1;
    s1 = seqa(1,2,rows(p0)/2);
    p0 = (q00[s1] + q00[s1+1])|(q01[s1] + q01[s1+1]);
    p1 = (q10[s1] + q10[s1+1])|(q11[s1] + q11[s1+1]);
  else;
    p0 = q00 + q01;
    p1 = q10 + q11;
  endif;
  ff = sumc(p0) + sumc(p1);
  p0 = p0./ff;
  p1 = p1./ff;
  fit[it] = ff;

it = it+1;
endo;
if minc(fit) .> 0;
 f = ln(fit);
else;
 f = -1000;
endif;

retp(-f);
endp;


proc clike(th);
local ths;
  ths = th|b1|sig1|p|q;
retp(marklike(ths));
endp;

proc mclike(th);
local lik;
lik = sumc(clike(th));
retp(lik);
endp;

proc glike(th);
local lik,p,q,th2;
p = th[5+nar];
q = th[6+nar];
p = exp(p)/(1+exp(p));
q = exp(q)/(1+exp(q));
th2 = th[1:4+nar]|p|q;
lik = marklike(th2);
retp(lik);
endp;

proc mglike(th);
local lik;
lik = sumc(glike(th));
retp(lik);
endp;

