/* This program applies CHP to test for Markov Switching in Hamilton's AR model on GNP */
/* test for switching in the mean AND variance */
/* Hamilton's original series: t = 135 for gnp82.dat */
/* Extended series: t = 239 for gnpc96.txt */
/* critical value and p-values are reported */
/* Table 3 in the resubmission */

new;

rndseed 2938438;

t = 135;
rho_b = 0.7;
itnb = 3000;
load gnp[]=c:/paper/werner/revision/application/hamilton/gnp82.dat;

gnp = 100*ln(gnp);
dat = gnp[2:t+1] - gnp[1:t];

nar = 4;         

nn = t-nar;

y = dat[1+nar:t];
x = dat[nar:t-1];

it = 2; 
do while it <= nar;
  x = x~dat[nar-it+1:t-it];
  it = it + 1; 
endo;

/* first estimate the AR under H0 */

xx = ones(nn,1)~x;

b0 = inv(xx'xx)*(xx'y);
u = y - xx*b0;
v0 = sqrt((u'u)/rows(u));
var0 = inv(xx'xx)*((xx.*u)'(xx.*u))*inv(xx'xx);
se0 = sqrt(diag(var0));

phi = b0[2:1+nar];      @AR cofficients@
mu0 = b0[1]/(1-sumc(phi));  @ mean of GNP growth @

/* get the 1st derivative of the log likelihood, use lt as prefix */

ltmu = u*(1-sumc(phi))/v0^2;

ltphi = u.*(x[.,1]-mu0)/v0^2;

for i(2,nar,1);
    ltphi = ltphi~(u.*(x[.,i]-mu0)/v0^2);
endfor;

ltsig2 = u^2/(2*v0^4)-1/(2*v0^2);

ltmx = ltmu~ltphi~ltsig2; @ TX(2+nar) matrix @

/* get the 2nd derivative of the log likelihood, use mt as prefix */

mtmu = -(1-sumc(phi))^2/v0^2;   @scalar@

mtmuphi = -u/v0^2+(1-sumc(phi))*(mu0-x[.,1])/v0^2;

for i(2,nar,1);
    mtmuphi = mtmuphi~(-u/v0^2+(1-sumc(phi))*(mu0-x[.,i])/v0^2);
endfor;

mtmusig2 = -u*(1-sumc(phi))/v0^4;

/* The 2nd derivative with respect to phi_i and phi_j is calculated in the loop for getting mu2t 
for i(1,nar,1);
    for j(1,nar,1);
        mtphi = -(mu0-x[i]).*(mu0-x[j])/v0^2;
    endfor;
endfor;

*/ 
mtphisig2 = u.*(mu0-x)/v0^4;

mtsig2 = 1/(2*v0^4)-u^2/(v0^6);

/* calculate our test statistic supTS and expTS */

h = rndn(100,2);


cv = zeros(200,100); @stores supTS critical values for each rho@

cv2 = zeros(200,100);    @stores expTS critical values for each rho@

ih = 1; 
do while ih <= 100;

hu = h[ih,.]/(h[ih,.]*h[ih,.]');   @h-vector uniformly over the unit sphere@
hv = hu[1] ~ zeros(1,nar) ~ hu[2];

ir = 1;           @iterations for rho@
rho = -rho_b;   @lower bound of rho@
do while rho <= rho_b + 0.01;

mu2t = zeros(nn,1);
xs = zeros(nn,6);

xs[2,.] = rho * ltmx[1,.];

mu2t[2] = hv * ltmx[2,.]'*xs[2,.] * hv';


for i(1,nn,1);
    mtphi = -(x[i,.]-mu0)'*(x[i,.]-mu0);    
    m = (mtmu~mtmuphi[i,.]~mtmusig2[i])|(mtmuphi[i,.]'~mtphi~mtphisig2[i,.]')|(mtmusig2[i]~mtphisig2[i,.]~mtsig2[i]);
    lx = ltmx[i,.]'ltmx[i,.];
    mu2t[i] = mu2t[i] + hv * (m + lx) * hv' / 2;
    
    if i > 2;
        xs[i,.] = rho * (xs[i-1,.] + ltmx[i-1,.]);
        mu2t[i] = mu2t[i] + hv * ltmx[i,.]' * xs[i,.] *hv';
    endif;
endfor;

gamma_e = 1/sqrt(t)*sumc(mu2t);
 
epsilont = mu2t-ltmx*inv(ltmx'ltmx)*ltmx'mu2t; @projection of mu2t on lt1@
esqe = meanc(epsilont^2);
tspe = gamma_e/sqrt(esqe);

if esqe<1e-5;
    cv[ir,ih] = 0;
    cv2[ir,ih] = 1;
else;
    cv[ir,ih] = 1/2*maxc(tspe|0)^2;    @supTS test statistic@ 
    cv2[ir,ih] = sqrt(2*pi)*exp((tspe-1)^2/2)*cdfn(tspe-1);  @expTS test statistic@
endif;

rho = rho+0.01;
ir = ir+1;
endo;
ih = ih+1;
endo;

supts = maxc(vec(cv[1:ir-1,.]));
expts = meanc(vec(cv2[1:ir-1,.]));

print /flush "cv";
print /flush cv;
print /flush "cv2";
print /flush cv2;
/* bootstrap the critical values */
supb = zeros(itnb,1);   @stores the bootstrapped supTS critical value@
expb = zeros(itnb,1);   @stores the bootstrapped expTS critical value@


for itb(1,itnb,1);

/* first simulate the series itnb(e.g., 3000) times according to ML estimators */
ys = zeros(t,1);
ys[1:nar] = meanc(y)+stdc(y)*rndn(nar,1);   @start from stationary distribution@

for i(nar+1,t,1);
    ys[i] = b0'(1|ys[i-1:i-nar])+v0*rndn(1,1);
endfor;

yb = ys[1+nar:t];
xb = ys[nar:t-1];

it = 2; 
do while it <= nar;
  xb = xb~ys[nar-it+1:t-it];
  it = it + 1; 
endo;


/* first estimate the AR under H0 */

xxb = ones(nn,1)~xb;

b0b = inv(xxb'xxb)*(xxb'yb);
ub = yb - xxb*b0b;
v0b = sqrt((ub'ub)/rows(ub));

phib = b0b[2:1+nar];      @AR cofficients@
mu0b = b0b[1]/(1-sumc(phib));  @ mean of GNP growth @

/* get the 1st derivative of the log likelihood, use lt as prefix */

ltmu = ub*(1-sumc(phib))/v0b^2;

ltphi = ub.*(xb[.,1]-mu0b)/v0b^2;

for i(2,nar,1);
    ltphi = ltphi~(ub.*(xb[.,i]-mu0b)/v0b^2);
endfor;

ltsig2 = ub^2/(2*v0b^4)-1/(2*v0b^2);

ltmx = ltmu~ltphi~ltsig2; @ TX(2+nar) matrix @

/* get the 2nd derivative of the log likelihood, use mt as prefix */

mtmu = -(1-sumc(phib))^2/v0b^2;   @scalar@

mtmuphi = -ub/v0b^2+(1-sumc(phib))*(mu0b-xb[.,1])/v0b^2;

for i(2,nar,1);
    mtmuphi = mtmuphi~(-ub/v0b^2+(1-sumc(phib))*(mu0b-xb[.,i])/v0b^2);
endfor;

mtmusig2 = -ub*(1-sumc(phib))/v0b^4;

/* The 2nd derivative with respect to phi_i and phi_j is calculated in the loop for getting mu2t 
for i(1,nar,1);
    for j(1,nar,1);
        mtphi = -(mu0-x[i]).*(mu0-x[j])/v0^2;
    endfor;
endfor;

*/ 
mtphisig2 = ub.*(mu0b-xb)/v0b^4;

mtsig2 = 1/(2*v0b^4)-ub^2/(v0b^6);

/* calculate our test statistic supTS and expTS */


cv = zeros(200,100); @stores supTS critical values for each rho and h@

cv2 = zeros(200,100);    @stores expTS critical values for each rho and h@

ih = 1; 
do while ih<=100;

hu = h[ih,.]/(h[ih,.]*h[ih,.]');   @h-vector uniformly over the unit sphere@
hv = hu[1] ~ zeros(1,nar) ~ hu[2];

ir = 1;           @iterations for rho@
rho = -rho_b;   @lower bound of rho@

do while rho<=rho_b + 0.01;

mu2t = zeros(nn,1);
xs = zeros(nn,6);

xs[2,.] = rho * ltmx[1,.];

mu2t[2] = hv * ltmx[2,.]'*xs[2,.] * hv';


for i(1,nn,1);
    mtphi = -(xb[i,.]-mu0b)'*(xb[i,.]-mu0b);    
    m = (mtmu~mtmuphi[i,.]~mtmusig2[i])|(mtmuphi[i,.]'~mtphi~mtphisig2[i,.]')|(mtmusig2[i]~mtphisig2[i,.]~mtsig2[i]);
    lx = ltmx[i,.]'ltmx[i,.];
    mu2t[i] = mu2t[i] + hv * (m + lx) * hv' / 2;
    
    if i > 2;
        xs[i,.] = rho * (xs[i-1,.] + ltmx[i-1,.]);
        mu2t[i] = mu2t[i] + hv * ltmx[i,.]' * xs[i,.] *hv';
    endif;
endfor;

gamma_e = 1/sqrt(t)*sumc(mu2t);

epsilont = mu2t-ltmx*inv(ltmx'ltmx)*ltmx'mu2t; @projection of mu2t on lt1@
esqe = meanc(epsilont^2);
tspe = gamma_e/sqrt(esqe);  

if esqe<1e-5;
    cv[ir,ih] = 0;
    cv2[ir,ih] = 1;
else;
    cv[ir,ih] = 1/2*maxc(tspe|0)^2;    @supTS test statistic@ 
    cv2[ir,ih] = sqrt(2*pi)*exp((tspe-1)^2/2)*cdfn(tspe-1);  @expTS test statistic@
endif;
rho = rho+0.01;
ir = ir+1;
endo;
ih = ih+1;
endo;


supb[itb] = maxc(vec(cv[1:ir-1,.]));
expb[itb] = meanc(vec(cv2[1:ir-1,.]));

endfor;
/*empirical critical values */

supb = sortc(supb,1);
expb = sortc(expb,1);

crtsup99=supb[floor(itnb*0.99)];
crtsup95=supb[floor(itnb*0.95)];
crtsup90=supb[floor(itnb*0.90)];   

crtexp99=expb[floor(itnb*0.99)];
crtexp95=expb[floor(itnb*0.95)];
crtexp90=expb[floor(itnb*0.90)];   

format /rd 8,3;
"Estimation Under Null of Linearity";
" Regression Parameters   Standard Error"
b0~se0;
"";
"Standard Deviation : " v0;
"";
"our test statstic ";
"supTS is " supts;
"expTS is " expts;

"Iteration of simulation is " itnb;
"supTS critical values are " crtsup99 crtsup95 crtsup90;
"expTS critical values are " crtexp99 crtexp95 crtexp90;
"supTS p-value is " meanc(supts.<supb);
"expTS p-value is " meanc(expts.<expb);

end;
