/* December 2011 */
/* This program estimates the GNP series under the linear null hypothesis */
/* AR(4) model */
/* Hamilton's original data, gnp82.dat with t = 135 */
/* Extended data, gnpc96.txt with t = 239 */
/* reports Heteroskedasticity-Robust standard errors as in Hansen and Garcia */
/* Table A2 in the resubmission */

new;
library cml;
cmlset;
rndseed 2938438;
__output = 0;
_cml_CovPar = 3;
t = 239;
rho_b = 0.7;
itnb = 3000;
load gnp[]=c:/paper/werner/revision/application/hamilton/gnpc96.txt;

gnp = 100*ln(gnp);
dat = gnp[2:t+1] - gnp[1:t];

nar = 4;         

nn = t-nar;

y = dat[1+nar:t];
x = dat[nar:t-1];

it = 2; 
do while it <= nar;
  x = x~dat[nar-it+1:t-it];
  it = it + 1; 
endo;

/* first estimate the AR under H0 */

xx = ones(nn,1)~x;

b0 = inv(xx'xx)*(xx'y);
u = y - xx*b0;
v0 = sqrt((u'u)/rows(u));
var0 = inv(xx'xx)*((xx.*u)'(xx.*u))*inv(xx'xx);
se0 = sqrt(diag(var0));

phi = b0[2:1+nar];      @AR cofficients@
mu0 = b0[1]/(1-sumc(phi));  @ mean of GNP growth @

st = ones(6,1);
{b,h,g,cov, ret} = cml(dat,0,&likhd,st );

"OLS";
" Regression Parameters   Standard Error"
b0~se0;
"";
"Standard Deviation : " v0;
"MLE";
b~sqrt(diag(cov));
end;

proc likhd(x,dat);
    local n, mu, phi, sig, lk;
    n   = rows(dat);
    
    mu = x[1];
    phi = x[2:5];
    sig = x[6];
    lk = zeros(n,1);
    for i(5,n,1);
        lk[i] = -ln(2*pi)/2 - ln(sig^2)/2 - (dat[i] - phi'*dat[i-1:i-4] - (1-sumc(phi))*mu)^2/(2*sig^2);
    endfor;
    
    retp(lk[5:n]);
    endp;
