/* This program checks the power of CHP test */
/* The dataset is the extended dataset */
/* test for switching in the mean AND the variance */
/* Report power */

/* 
critical values are from ha_mean_var.g 
supTS critical values are    5.782    3.809    3.103 
expTS critical values are    2.653    1.564    1.242 
*/

new;

rndseed 2938438;

t = 239;
rho_b = 0.7;    @upper bound for rho@
itnb = 3000;    @number of iterations@
ksi = {1 0};    @values the Markov Chain takes@
nar = 4;        @AR(4)@

/* Markov Switching Estimation Results using Hamilton's program */

phih = {0.3054, 0.2103, -0.1204, -0.0528 };      @AR cofficients@
muh = 0.7131 ;  
mud  = 0.0991;

sig0 = 1.1052;
sig1 = 0.4212;
p  = 0.9560;
q = 0.9669;


sev = sig1 ~ sig0;  @collects the standard error in two regimes@

trm = p ~(1-p) | (1-q) ~ q;    @transition matrix@

supb = zeros(itnb,1);       @collects supTS@
expb = zeros(itnb,1);       @collects expTS@

for it(1,itnb,1);

s0 = rndu(1,1);

s0 = 1 * (s0 < (1-q)/(2-p-q)) + 2 * (s0 > (1-q)/(2-p-q));

st = markov(trm, t+200,s0); 

alt=ksi[s0]~ ksi*st;

alt = alt';

altv = sev[s0] ~ sev*st;

altv = altv';

/* Simulate the data according to the above estimation results */

e = rndn(t+200,1) .* altv;  @error term@

uh = zeros(t+200,1);

for i(nar+1,t+200,1);
    uh[i] = phih'*uh[i-1:i-nar] + e[i];
endfor;

dat = muh + mud * alt + uh;

nn = t-nar;

y = dat[201+nar:t+200];
x = dat[nar+200:t+199];

in = 2; 
do while in <= nar;
  x = x ~ dat[nar-in+201:t-in+200];
  in = in + 1; 
endo;

/* first estimate the AR under H0 */

xx = ones(nn,1)~x;

b0 = inv(xx'xx)*(xx'y);
u = y - xx*b0;
v0 = sqrt((u'u)/rows(u));
var0 = inv(xx'xx)*((xx.*u)'(xx.*u))*inv(xx'xx);
se0 = sqrt(diag(var0));

phi = b0[2:1+nar];      @AR cofficients@
mu0 = b0[1]/(1-sumc(phi));  @ mean of GNP growth @

/* get the 1st derivative of the log likelihood, use lt as prefix */

ltmu = u*(1-sumc(phi))/v0^2;

ltphi = u.*(x[.,1]-mu0)/v0^2;

for i(2,nar,1);
    ltphi = ltphi~(u.*(x[.,i]-mu0)/v0^2);
endfor;

ltsig2 = u^2/(2*v0^4)-1/(2*v0^2);

ltmx = ltmu~ltphi~ltsig2;

/* get the 2nd derivative of the log likelihood, use mt as prefix */

mtmu = -(1-sumc(phi))^2/v0^2;   @scalar@

mtmuphi = -u/v0^2+(1-sumc(phi))*(mu0-x[.,1])/v0^2;

for i(2,nar,1);
    mtmuphi = mtmuphi~(-u/v0^2+(1-sumc(phi))*(mu0-x[.,i])/v0^2);
endfor;

mtmusig2 = -u*(1-sumc(phi))/v0^4;


mtphisig2 = u.*(mu0-x)/v0^4;

mtsig2 = 1/(2*v0^4)-u^2/(v0^6);

/* calculate our test statistic supTS and expTS */

h = rndn(100,2);


cv = zeros(200,100); @stores supTS critical values for each rho@

cv2 = zeros(200,100);    @stores expTS critical values for each rho@

ih = 1; 
do while ih <= 100;

hu = h[ih,.]/(h[ih,.]*h[ih,.]');   @h-vector uniformly over the unit sphere@
hv = hu[1] ~ zeros(1,nar) ~ hu[2];

rho = -rho_b;   @lower bound of rho@
ir = 1;           @iterations for rho@

do while rho<=rho_b+0.01;

mu2t = zeros(nn,1);
xs = zeros(nn,6);

xs[2,.] = rho * ltmx[1,.];

mu2t[2] = hv * ltmx[2,.]'*xs[2,.] * hv';


for i(1,nn,1);
    mtphi = -(x[i,.]-mu0)'*(x[i,.]-mu0);    
    m = (mtmu~mtmuphi[i,.]~mtmusig2[i])|(mtmuphi[i,.]'~mtphi~mtphisig2[i,.]')|(mtmusig2[i]~mtphisig2[i,.]~mtsig2[i]);
    lx = ltmx[i,.]'ltmx[i,.];
    mu2t[i] = mu2t[i] + hv * (m + lx) * hv' / 2;
    
    if i > 2;
        xs[i,.] = rho * (xs[i-1,.] + ltmx[i-1,.]);
        mu2t[i] = mu2t[i] + hv * ltmx[i,.]' * xs[i,.] *hv';
    endif;
endfor;

gamma_e = 1/sqrt(t)*sumc(mu2t);

epsilont = mu2t-ltmx*inv(ltmx'ltmx)*ltmx'mu2t; @projection of mu2t on lt1@
esqe = meanc(epsilont^2);
tspe = gamma_e/sqrt(esqe);

if esqe<1e-5;
    cv[ir,ih] = 0;
    cv2[ir,ih] = 1;
else;
    cv[ir,ih] = 1/2*maxc(tspe|0)^2;    @supTS test statistic@ 
    cv2[ir,ih] = sqrt(2*pi)*exp((tspe-1)^2/2)*cdfn(tspe-1);  @expTS test statistic@
endif;

rho = rho + 0.01;
ir = ir + 1;
endo;
ih = ih+1;
endo;


supb[it] = maxc(vec(cv[1:ir-1,.]));
expb[it] = meanc(vec(cv2[1:ir-1,.]));

endfor;


/* powers */

supb = sortc(supb,1);
expb = sortc(expb,1);

                
"Iteration of simulation is " itnb;


crtsup99 = 5.782;
crtsup95 = 3.809;
crtsup90 = 3.103;

                
crtexp99 = 2.653;
crtexp95 = 1.564;
crtexp90 = 1.242;

format /rd 8,3;
"supTS critical values are " crtsup99 crtsup95 crtsup90;
"expTS critical values are " crtexp99 crtexp95 crtexp90;

"Powers are ";
"supTS at 1% " 100*meanc(supb .> crtsup99);
"expTS at 1% " 100*meanc(expb .> crtexp99);
"";
"supTS at 5% " 100*meanc(supb .> crtsup95);
"expTS at 5% " 100*meanc(expb .> crtexp95);
"";
"supTS at 10% " 100*meanc(supb .> crtsup90);
"expTS at 10% " 100*meanc(expb .> crtexp90);
"";


end;

/* The procedure to get a Markov chain */

proc(1)=markov(trs,t,s0);	
/*  trs is the transition matrix
     t is the number of periods to simulate
     s0 is the initial state   */

local r,c,x,s,state,cum,k,ppi,ks;
r=rows(trs);
c=cols(trs);
x=rndu(t-1,1);
s=zeros(r,1);
s[s0]=1;

state=zeros(r,t-1); @state is a matrix recording the number of the realized state at time t @

cum=trs*upmat(ones(r,c));

for k(1,t-1,1);
	state[.,k]=s;
	ppi=0~s'*cum;
	s=((x[k].<=ppi[2:r+1]).*(x[k].>ppi[1:r]))';
endfor;

@
chain=ksi*state; chain is a row vector which stores the (T-1) period simulations@

retp(state);
endp;
